]> git.sesse.net Git - nageru/blobdiff - mixer.h
Re-run IWYU, again with lots of manual cleanup.
[nageru] / mixer.h
diff --git a/mixer.h b/mixer.h
index f4ea5c9cd6b41b57cbbec2bae3ce867100aef222..bf53c2482e23362e0cba1ff589cbf72d76d78dd0 100644 (file)
--- a/mixer.h
+++ b/mixer.h
@@ -5,19 +5,39 @@
 
 #include <epoxy/gl.h>
 #undef Success
+#include <stdbool.h>
+#include <stdint.h>
+
 #include <movit/effect_chain.h>
 #include <movit/flat_input.h>
+#include <condition_variable>
+#include <cstddef>
 #include <functional>
-
-#include "bmusb.h"
+#include <memory>
+#include <mutex>
+#include <string>
+#include <thread>
+#include <vector>
+
+#include "bmusb/bmusb.h"
+#include "ebu_r128_proc.h"
 #include "h264encode.h"
+#include "httpd.h"
 #include "pbo_frame_allocator.h"
 #include "ref_counted_frame.h"
 #include "ref_counted_gl_sync.h"
-#include "theme.h"
 #include "resampler.h"
+#include "theme.h"
+#include "timebase.h"
 
-#define NUM_CARDS 2
+class H264Encoder;
+class QSurface;
+namespace movit {
+class Effect;
+class EffectChain;
+class FlatInput;
+class ResourcePool;
+}  // namespace movit
 
 namespace movit {
 class YCbCrInput;
@@ -28,7 +48,7 @@ class QSurfaceFormat;
 class Mixer {
 public:
        // The surface format is used for offscreen destinations for OpenGL contexts we need.
-       Mixer(const QSurfaceFormat &format);
+       Mixer(const QSurfaceFormat &format, unsigned num_cards);
        ~Mixer();
        void start();
        void quit();
@@ -78,19 +98,29 @@ public:
                output_channel[output].set_frame_ready_callback(callback);
        }
 
+       typedef std::function<void(float, float, float, float, float)> audio_level_callback_t;
+       void set_audio_level_callback(audio_level_callback_t callback)
+       {
+               audio_level_callback = callback;
+       }
+
        std::vector<std::string> get_transition_names()
        {
-               return theme->get_transition_names(frame / 60.0);
+               return theme->get_transition_names(pts());
        }
 
 private:
-       void bm_frame(int card_index, uint16_t timecode,
+       void bm_frame(unsigned card_index, uint16_t timecode,
                FrameAllocator::Frame video_frame, size_t video_offset, uint16_t video_format,
                FrameAllocator::Frame audio_frame, size_t audio_offset, uint16_t audio_format);
        void place_rectangle(movit::Effect *resample_effect, movit::Effect *padding_effect, float x0, float y0, float x1, float y1);
        void thread_func();
        void subsample_chroma(GLuint src_tex, GLuint dst_dst);
        void release_display_frame(DisplayFrame *frame);
+       double pts() { return double(pts_int) / TIMEBASE; }
+
+       HTTPD httpd;
+       unsigned num_cards;
 
        QSurface *mixer_surface, *h264_encoder_surface;
        std::unique_ptr<movit::ResourcePool> resource_pool;
@@ -102,7 +132,7 @@ private:
        // Effects part of <display_chain>. Owned by <display_chain>.
        movit::FlatInput *display_input;
 
-       int frame = 0;
+       int64_t pts_int = 0;  // In TIMEBASE units.
 
        std::mutex bmusb_mutex;
        struct CaptureCard {
@@ -113,15 +143,21 @@ private:
                QSurface *surface;
                QOpenGLContext *context;
 
-               bool new_data_ready = false;  // Whether new_frame contains anything.
+               bool new_data_ready = false;  // Whether new_frame and new_frame_audio contains anything.
+               bool should_quit = false;
                RefCountedFrame new_frame;
                GLsync new_data_ready_fence;  // Whether new_frame is ready for rendering.
+               std::vector<float> new_frame_audio;
                std::condition_variable new_data_ready_changed;  // Set whenever new_data_ready is changed.
-               Resampler *resampler = nullptr;
+               unsigned dropped_frames = 0;  // Before new_frame.
+
+               std::mutex audio_mutex;
+               std::unique_ptr<Resampler> resampler;  // Under audio_mutex.
+               int last_timecode = -1;  // Unwrapped.
        };
-       CaptureCard cards[NUM_CARDS];  // protected by <bmusb_mutex>
+       CaptureCard cards[MAX_CARDS];  // protected by <bmusb_mutex>
 
-       RefCountedFrame bmusb_current_rendering_frame[NUM_CARDS];
+       RefCountedFrame bmusb_current_rendering_frame[MAX_CARDS];
 
        class OutputChannel {
        public:
@@ -144,6 +180,12 @@ private:
 
        std::thread mixer_thread;
        bool should_quit = false;
+
+       audio_level_callback_t audio_level_callback = nullptr;
+       Ebu_r128_proc r128;
+
+       // TODO: Implement oversampled peak detection.
+       float peak = 0.0f;
 };
 
 extern Mixer *global_mixer;